@@ -234,12 +234,15 @@ static short feh_is_kp(fehkey *key, unsigned int state, unsigned int sym, unsign
 
 void feh_event_invoke_action(winwidget winwid, unsigned char action)
 {
+	struct stat st;
 	if (opt.actions[action]) {
 		if (opt.slideshow) {
 			feh_action_run(FEH_FILE(winwid->file->data), opt.actions[action], winwid);
 
 			if (opt.hold_actions[action])
 				feh_reload_image(winwid, 1, 1);
+			else if (stat(FEH_FILE(winwid->file->data)->filename, &st) == -1)
+				feh_filelist_image_remove(winwid, 0);
 			else
 				slideshow_change_image(winwid, SLIDE_NEXT, 1);
 
@@ -266,6 +269,30 @@ void feh_event_invoke_action(winwidget winwid, unsigned char action)
 	return;
 }
 
+void feh_event_handle_stdin()
+{
+	char stdin_buf[2];
+	KeySym keysym = NoSymbol;
+	if (read(STDIN_FILENO, &stdin_buf, 1) == -1) {
+		if (control_via_stdin)
+			weprintf("reading a command from stdin failed");
+		return;
+	}
+	stdin_buf[1] = '\0';
+
+	if (stdin_buf[0] == ' ')
+		keysym = XK_space;
+	else if (stdin_buf[0] == '\n')
+		keysym = XK_Return;
+	else if ((stdin_buf[0] == '\b') || (stdin_buf[0] == 127))
+		keysym = XK_BackSpace;
+	else
+		keysym = XStringToKeysym(stdin_buf);
+
+	if (window_num)
+		feh_event_handle_generic(windows[0], 0, keysym, 0);
+}
+
 void feh_event_handle_keypress(XEvent * ev)
 {
 	int state;
@@ -311,57 +338,6 @@ void feh_event_handle_keypress(XEvent * ev)
 	if (winwid == NULL)
 		return;
 
-	if (winwid->caption_entry) {
-		switch (keysym) {
-		case XK_Return:
-			if (state & ControlMask) {
-				/* insert actual newline */
-				ESTRAPPEND(FEH_FILE(winwid->file->data)->caption, "\n");
-				winwidget_render_image_cached(winwid);
-			} else {
-				/* finish caption entry, write to captions file */
-				FILE *fp;
-				char *caption_filename;
-				caption_filename =
-					build_caption_filename(FEH_FILE(winwid->file->data), 1);
-				winwid->caption_entry = 0;
-				winwidget_render_image_cached(winwid);
-				XFreePixmap(disp, winwid->bg_pmap_cache);
-				winwid->bg_pmap_cache = 0;
-				fp = fopen(caption_filename, "w");
-				if (!fp) {
-					eprintf("couldn't write to captions file %s:", caption_filename);
-					return;
-				}
-				fprintf(fp, "%s", FEH_FILE(winwid->file->data)->caption);
-				free(caption_filename);
-				fclose(fp);
-			}
-			break;
-		case XK_Escape:
-			/* cancel, revert caption */
-			winwid->caption_entry = 0;
-			free(FEH_FILE(winwid->file->data)->caption);
-			FEH_FILE(winwid->file->data)->caption = NULL;
-			winwidget_render_image_cached(winwid);
-			XFreePixmap(disp, winwid->bg_pmap_cache);
-			winwid->bg_pmap_cache = 0;
-			break;
-		case XK_BackSpace:
-			/* backspace */
-			ESTRTRUNC(FEH_FILE(winwid->file->data)->caption, 1);
-			winwidget_render_image_cached(winwid);
-			break;
-		default:
-			if (isascii(keysym)) {
-				/* append to caption */
-				ESTRAPPEND_CHAR(FEH_FILE(winwid->file->data)->caption, keysym);
-				winwidget_render_image_cached(winwid);
-			}
-			break;
-		}
-		return;
-	}
 	feh_event_handle_generic(winwid, state, keysym, 0);
 }
 
@@ -504,6 +480,58 @@ fehkey *feh_str_to_kb(char *action)
 void feh_event_handle_generic(winwidget winwid, unsigned int state, KeySym keysym, unsigned int button) {
 	int curr_screen = 0;
 
+	if (winwid->caption_entry && (keysym != NoSymbol)) {
+		switch (keysym) {
+		case XK_Return:
+			if (state & ControlMask) {
+				/* insert actual newline */
+				ESTRAPPEND(FEH_FILE(winwid->file->data)->caption, "\n");
+				winwidget_render_image_cached(winwid);
+			} else {
+				/* finish caption entry, write to captions file */
+				FILE *fp;
+				char *caption_filename;
+				caption_filename =
+					build_caption_filename(FEH_FILE(winwid->file->data), 1);
+				winwid->caption_entry = 0;
+				winwidget_render_image_cached(winwid);
+				XFreePixmap(disp, winwid->bg_pmap_cache);
+				winwid->bg_pmap_cache = 0;
+				fp = fopen(caption_filename, "w");
+				if (!fp) {
+					eprintf("couldn't write to captions file %s:", caption_filename);
+					return;
+				}
+				fprintf(fp, "%s", FEH_FILE(winwid->file->data)->caption);
+				free(caption_filename);
+				fclose(fp);
+			}
+			break;
+		case XK_Escape:
+			/* cancel, revert caption */
+			winwid->caption_entry = 0;
+			free(FEH_FILE(winwid->file->data)->caption);
+			FEH_FILE(winwid->file->data)->caption = NULL;
+			winwidget_render_image_cached(winwid);
+			XFreePixmap(disp, winwid->bg_pmap_cache);
+			winwid->bg_pmap_cache = 0;
+			break;
+		case XK_BackSpace:
+			/* backspace */
+			ESTRTRUNC(FEH_FILE(winwid->file->data)->caption, 1);
+			winwidget_render_image_cached(winwid);
+			break;
+		default:
+			if (isascii(keysym)) {
+				/* append to caption */
+				ESTRAPPEND_CHAR(FEH_FILE(winwid->file->data)->caption, keysym);
+				winwidget_render_image_cached(winwid);
+			}
+			break;
+		}
+		return;
+	}
+
 	if (feh_is_kp(&keys.next_img, state, keysym, button)) {
 		if (opt.slideshow)
 			slideshow_change_image(winwid, SLIDE_NEXT, 1);
@@ -743,7 +771,7 @@ void feh_event_handle_generic(winwidget winwid, unsigned int state, KeySym keysy
 	else if (feh_is_kp(&keys.size_to_image, state, keysym, button)) {
 		winwidget_size_to_image(winwid);
 	}
-	else if (feh_is_kp(&keys.toggle_menu, state, keysym, button)) {
+	else if (!opt.no_menus && feh_is_kp(&keys.toggle_menu, state, keysym, button)) {
 		winwidget_show_menu(winwid);
 	}
 	else if (feh_is_kp(&keys.close, state, keysym, button)) {
