@@ -32,11 +32,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "events.h"
 #include "signals.h"
 #include "wallpaper.h"
+#include <termios.h>
 
 char **cmdargv = NULL;
 int cmdargc = 0;
 char *mode = NULL;
 
+struct termios old_term_settings;
+int control_via_stdin = 0;
+
 int main(int argc, char **argv)
 {
 	atexit(feh_clean_exit);
@@ -107,6 +111,31 @@ int feh_main_iteration(int block)
 		fdsize = xfd + 1;
 		pt = feh_get_time();
 		first = 0;
+		/*
+		 * Only accept commands from stdin if
+		 * - stdin is a terminal (otherwise it's probably used as an image / filelist)
+		 * - we aren't running in multiwindow mode (cause it's not clear which
+		 *   window commands should be applied to in that case)
+		 * - we're in the same process group as stdin, AKA we're not running
+		 *   in the background. Background processes are stopped with SIGTTOU
+		 *   if they try to write to stdout or change terminal attributes. They
+		 *   also don't get input from stdin anyway.
+		 */
+		if (isatty(STDIN_FILENO) && !opt.multiwindow && getpgrp() == (tcgetpgrp(STDIN_FILENO))) {
+			control_via_stdin = 1;
+			struct termios ctrl;
+			if (tcgetattr(STDIN_FILENO, &old_term_settings) == -1)
+				eprintf("tcgetattr failed");
+			if (tcgetattr(STDIN_FILENO, &ctrl) == -1)
+				eprintf("tcgetattr failed");
+			ctrl.c_iflag &= ~(PARMRK | ISTRIP
+					| INLCR | IGNCR | IXON);
+			ctrl.c_lflag &= ~(ECHO | ICANON | IEXTEN);
+			ctrl.c_cflag &= ~(CSIZE | PARENB);
+			ctrl.c_cflag |= CS8;
+			if (tcsetattr(STDIN_FILENO, TCSANOW, &ctrl) == -1)
+				eprintf("tcsetattr failed");
+		}
 	}
 
 	/* Timers */
@@ -127,6 +156,8 @@ int feh_main_iteration(int block)
 
 	FD_ZERO(&fdset);
 	FD_SET(xfd, &fdset);
+	if (control_via_stdin)
+		FD_SET(STDIN_FILENO, &fdset);
 
 	/* Timers */
 	ft = first_timer;
@@ -170,6 +201,8 @@ int feh_main_iteration(int block)
 				   in that */
 				feh_handle_timer();
 			}
+			else if (count && (FD_ISSET(0, &fdset)))
+				feh_event_handle_stdin();
 		}
 	} else {
 		/* Don't block if there are events in the queue. That's a bit rude ;-) */
@@ -181,6 +214,8 @@ int feh_main_iteration(int block)
 					&& ((errno == ENOMEM) || (errno == EINVAL)
 						|| (errno == EBADF)))
 				eprintf("Connection to X display lost");
+			else if (count && (FD_ISSET(0, &fdset)))
+				feh_event_handle_stdin();
 		}
 	}
 	if (window_num == 0)
@@ -199,6 +234,10 @@ void feh_clean_exit(void)
 	if(disp)
 		XCloseDisplay(disp);
 
+	if (control_via_stdin && isatty(STDIN_FILENO))
+		if (tcsetattr(STDIN_FILENO, TCSANOW, &old_term_settings) == -1)
+			eprintf("tcsetattr failed");
+
 	if (opt.filelistfile)
 		feh_write_filelist(filelist, opt.filelistfile);
 
